@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package AWS::SNS::Verify;
-$AWS::SNS::Verify::VERSION = '0.0101';
+$AWS::SNS::Verify::VERSION = '0.0102';
 use JSON;
 use HTTP::Tiny;
 use MIME::Base64;
@@ -9,6 +9,7 @@ use Moo;
 use Ouch;
 use Crypt::OpenSSL::RSA;
 use Crypt::OpenSSL::X509;
+use URI::URL;
 
 has body => (
     is          => 'ro',
@@ -42,9 +43,16 @@ has certificate => (
     }
 );
 
+has validate_signing_cert_url => (
+    is      => 'ro',
+    lazy    => 1,
+    default => 1,
+);
+
 sub fetch_certificate {
     my $self = shift;
-    my $response = HTTP::Tiny->new->get($self->message->{SigningCertURL});
+    my $url = $self->valid_cert_url($self->message->{SigningCertURL});
+    my $response = HTTP::Tiny->new->get($url);
     if ($response->{success}) {
         return $response->{content};
     }
@@ -87,6 +95,34 @@ sub verify {
     return 1;
 }
 
+# See also:
+# https://github.com/aws/aws-php-sns-message-validator/blob/master/src/MessageValidator.php#L22
+sub valid_cert_url {
+    my $self = shift;
+    my ($url_string) = @_;
+    $url_string ||= '';
+
+    return $url_string unless $self->validate_signing_cert_url;
+
+    my $url = URI::URL->new($url_string);
+    unless ( $url->can('host') ) {
+        ouch 'Bad SigningCertURL', "The SigningCertURL ($url_string) isn't a valid URL", $self;
+    }
+    my $host = $url->host;
+
+    # Match all regional SNS endpoints, e.g.
+    # sns.<region>.amazonaws.com        (AWS)
+    # sns.us-gov-west-1.amazonaws.com   (AWS GovCloud)
+    # sns.cn-north-1.amazonaws.com.cn   (AWS China)
+    my $dot = qr/\./;
+    my $region = qr/[a-zA-Z0-9-]+/;
+    unless ($host =~ /^ sns $dot $region $dot amazonaws $dot com(\.cn)? $/x) {
+        ouch 'Bad SigningCertURL', "The SigningCertURL ($url_string) isn't an Amazon endpoint", $self;
+    }
+
+    return $url_string;
+}
+
 
 =head1 NAME
 
@@ -94,7 +130,7 @@ AWS::SNS::Verify - Verifies authenticity of SNS messages.
 
 =head1 VERSION
 
-version 0.0101
+version 0.0102
 
 =head1 SYNOPSIS
 
@@ -140,13 +176,21 @@ Required. JSON string posted by AWS SNS. Looks like:
 
 =item certificate_string
 
-By default AWS::SNS::Verify will fetch the certificate string by issuing an HTTP GET request to C<SigningCertURL>. If you wish to use a cached version, then pass it in.
+By default AWS::SNS::Verify will fetch the certificate string by issuing an HTTP GET request to C<SigningCertURL>. The SigningCertURL in the message must be a AWS SNS endpoint.
+
+If you wish to use a cached version, then pass it in.
+
+=item validate_signing_cert_url (default: true)
+
+If you're using a fake SNS server in your local test environment, the SigningCertURL won't be an AWS endpoint. If so, set validate_signing_cert_url to 0.
+
+Don't ever do this in any kind of Production environment.
 
 =back
 
 =head2 verify
 
-Returns a 1 on success, or an L<Ouch> on a failure.
+Returns a 1 on success, or die with an L<Ouch> on a failure.
 
 =head2 message
 
@@ -157,6 +201,23 @@ Returns a hash reference of the decoded L<body> that was passed in to the constr
 If you want to cache the certificate in a local cache, then get it using this method.
 
 
+=head2 decode_signature
+
+You should never need to call this, it decodes the base64 signature.
+
+=head2 fetch_certificate
+
+You should never need to call this, it fetches the signing certificate.
+
+
+=head2 generate_signature_string
+
+You should never need to call this, it generates the signature string required to verify the request.
+
+=head2 valid_cert_url
+
+You should never need to call this, it checks the validity of the certificate signing URL per L<https://github.com/aws/aws-php-sns-message-validator/blob/master/src/MessageValidator.php#L22>
+
 =head1 REQUIREMENTS
 
 Requires Perl 5.12 or higher and these modules:
